'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var Koa = require('koa');
var Router = require('koa-router');
var cheerio = require('cheerio');
var fs = require('fs');
var url = require('url');
var reactRouterDom = require('react-router-dom');
var server = require('react-dom/server');
var path = require('path');
var send = _interopDefault(require('koa-send'));
var history = require('history');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const DEFAULT_WEBPACK_SERVER_PORT = 8080;
const DEFAULT_DEV_SERVER_PORT = 3030;
const DEFAULT_CLIENT_PUBLIC_PATH = '/static/';
const DEFAULT_SERVER_PUBLIC_PATH = '/';
const rootDirectory = process.cwd();
function getConfig() {
    const htmlFilename = 'client.html';
    const staticDirName = 'static';
    const buildDirName = 'build';
    return {
        clientChunkName: 'app.js',
        serverChunkName: 'server.js',
        webpackServerPort: DEFAULT_WEBPACK_SERVER_PORT,
        serverPort: DEFAULT_DEV_SERVER_PORT,
        clientPublicPath: DEFAULT_CLIENT_PUBLIC_PATH,
        serverPublicPath: DEFAULT_SERVER_PUBLIC_PATH,
        htmlTemplatePath: path.join(rootDirectory, 'src/index.html'),
        htmlFilename,
        htmlPath: path.join(rootDirectory, staticDirName, 'client.html'),
        buildDirName,
        staticDirName,
        buildDirectory: path.join(rootDirectory, buildDirName),
        staticDirectory: path.join(rootDirectory, staticDirName),
    };
}

const emptyProps = {};
const Container = ({ routes, matchedIndex, pageProps, AppContainer }) => {
    return (React.createElement(AppContainer, Object.assign({}, pageProps),
        React.createElement(reactRouterDom.Switch, null, routes.map((_a, index) => {
            var { component } = _a, routeProps = __rest(_a, ["component"]);
            const Page = component;
            const initialProps = index === matchedIndex
                ? pageProps
                : emptyProps;
            return (React.createElement(reactRouterDom.Route, Object.assign({ key: index }, routeProps, { render: props => (React.createElement(Page, Object.assign({}, props, initialProps))) })));
        }))));
};

const config = getConfig();
const isDev = process.env.NODE_ENV === 'development';
class Server {
    constructor(opts) {
        this.port = config.serverPort;
        this.clientChunkPath = new url.URL(config.clientChunkName, `http://localhost:${config.webpackServerPort}${config.clientPublicPath}`);
        this.container = opts.container;
        this.AppContainer = opts.AppContainer || React.Fragment;
        this.routes = opts.routes;
        const htmlPath = isDev ? config.htmlTemplatePath : config.htmlPath;
        this.originalHTML = fs.readFileSync(htmlPath, 'utf-8');
    }
    start() {
        const app = new Koa();
        const router = new Router();
        if (!isDev) {
            app.use(this.serveFiles);
        }
        router.get('*', this.handleRequest.bind(this));
        app.use(router.routes());
        app.listen(this.port, () => {
            console.log('Server listen on: http://localhost:' + this.port);
        });
    }
    handleRequest(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            const routes = this.routes;
            const pathname = ctx.url;
            const matchedIndex = routes.findIndex(route => {
                return !!reactRouterDom.matchPath(pathname, route);
            });
            if (matchedIndex !== -1) {
                const AppContainer = this.AppContainer;
                const matchedRoute = routes[matchedIndex];
                const pageProps = yield this.getInitialProps(AppContainer, pathname, matchedRoute);
                const content = server.renderToString(React.createElement(reactRouterDom.StaticRouter, { location: pathname, context: {} },
                    React.createElement(Container, { routes: routes, matchedIndex: matchedIndex, pageProps: pageProps, AppContainer: AppContainer })));
                ctx.body = this.renderHTML(content, pageProps);
            }
            else {
                ctx.body = 'Page not found';
            }
        });
    }
    serveFiles(ctx, next) {
        return __awaiter(this, void 0, void 0, function* () {
            const staticDirName = config.staticDirName;
            const staticDirectory = config.staticDirectory;
            const path = ctx.path;
            if (path.startsWith(`/${staticDirName}/`)) {
                yield send(ctx, path.replace(`/${staticDirName}/`, ''), { root: staticDirectory });
            }
            else {
                yield next();
            }
        });
    }
    renderHTML(content, pageProps) {
        const $ = cheerio.load(this.originalHTML);
        $(this.container).html(content);
        $('head').append(`
      <script type='text/javascript'>
          __APP_DATA__='${encodeURIComponent(JSON.stringify({ pageProps }))}'
      </script>
    `);
        if (isDev) {
            $('body').append(`
        <script type='text/javascript' src='${this.clientChunkPath}'></script>
      `);
        }
        return $.html();
    }
    getInitialProps(AppContainer, pathname, matchedRoute) {
        return __awaiter(this, void 0, void 0, function* () {
            const { getInitialProps } = AppContainer;
            if (getInitialProps) {
                return yield getInitialProps({ pathname, route: matchedRoute });
            }
            return {};
        });
    }
}
function render(opts) {
    const server = new Server(opts);
    server.start();
}

Object.keys(reactRouterDom).forEach(function (key) { exports[key] = reactRouterDom[key]; });
Object.keys(history).forEach(function (key) { exports[key] = history[key]; });
exports.render = render;
